#include "../unity/unity.h"
#include <stdlib.h>
#include <string.h>

/* Old implementation from the solution of exercise sheet 2:

  int fib_fast(int n) {
    if (n >= 5000) {
      return fib_slow(n);
    }
    int cache[5000];
    cache[0] = 0;
    cache[1] = 1;
    for (int i = 2; i <= n; ++i) {
      cache[i] = cache[i - 1] + cache[i - 2];
    }
    return cache[n];
  }

*/

/** Definitions ***************************************************************/

int fib_fast(int n) {
  if (n == 0) {
    return 0;
  }
  int *cache = malloc((n + 1) * sizeof(int));
  cache[0] = 0;
  cache[1] = 1;
  for (int i = 2; i <= n; ++i) {
    cache[i] = cache[i - 1] + cache[i - 2];
  }
  int fib_n = cache[n];
  free(cache);
  return fib_n;
}

char *repeat(size_t n, char *s) {
  size_t len = strlen(s);
  char *s_out = malloc((len * n + 1) * sizeof(char));
  for (size_t i = 0; i < len * n; ++i) {
    s_out[i] = s[i % len];
  }
  s_out[len * n] = 0;
  return s_out;
}

char *find_quoted(char *s) {
  char *begin = NULL;
  char *end = NULL;
  char *p = s;
  while (*p != 0) {
    if (*p == '"') {
      if (begin == NULL) {
        begin = p + 1;
      } else {
        end = p;
        break;
      }
    }
    p++;
  }
  if (end == NULL) {
    return NULL;
  }

  size_t len = end - begin;
  char *result = malloc((len + 1) * sizeof(char));
  for (size_t i = 0; i < len; ++i) {
    result[i] = begin[i];
  }
  result[len] = 0;
  return result;
}

char *join(char **strings, size_t num_strings, char *separator) {
  size_t len = 0;
  if (num_strings > 0) {
    len += strlen(separator) * (num_strings - 1);
  }
  for (size_t i = 0; i < num_strings; ++i) {
    len += strlen(strings[i]);
  }
  char *s_out = malloc((len + 1) * sizeof(char));
  char *p = s_out;
  for (size_t i = 0; i < num_strings; ++i) {
    for (char *s = strings[i]; *s != 0; ++s) {
      *(p++) = *s;
    }
    if (i < num_strings - 1) {
      for (char *s = separator; *s != 0; ++s) {
        *(p++) = *s;
      }
    }
  }
  *p = 0;
  return s_out;
}

/** Tests *********************************************************************/

void test_fib_fast_0(void) { TEST_ASSERT_EQUAL_INT(0, fib_fast(0)); }

void test_fib_fast_1(void) { TEST_ASSERT_EQUAL_INT(1, fib_fast(1)); }

void test_fib_fast_10(void) { TEST_ASSERT_EQUAL_INT(55, fib_fast(10)); }

void test_repeat(void) {
  char *s = repeat(3, "foo");
  TEST_ASSERT_EQUAL_STRING("foofoofoo", s);
  free(s);
}

void test_join_empty() {
  char *strings[] = {};
  char *s = join(strings, 0, ", ");
  TEST_ASSERT_EQUAL_STRING("", s);
  free(s);
}

void test_join_single() {
  char *strings[] = {"foo"};
  char *s = join(strings, 1, ", ");
  TEST_ASSERT_EQUAL_STRING("foo", s);
  free(s);
}

void test_join_many() {
  char *strings[] = {"Help!", "I'm split into", "multiple", "strings!"};
  char *expected = "Help! * I'm split into * multiple * strings!";
  char *actual = join(strings, 4, " * ");
  TEST_ASSERT_EQUAL_STRING(expected, actual);
  free(actual);
}

void test_find_quoted_1() {
  char *s = find_quoted("foo \"bar\" foo");
  TEST_ASSERT_EQUAL_STRING("bar", s);
  free(s);
}

void test_find_quoted_2() {
  char *s = find_quoted("foo \"bar foo");
  TEST_ASSERT_EQUAL_STRING(NULL, s);
}

void test_find_quoted_3() {
  char *s = find_quoted("foo bar foo");
  TEST_ASSERT_EQUAL_STRING(NULL, s);
}

/** Main **********************************************************************/

void setUp(void) {}
void tearDown(void) {}

int main(void) {
  UNITY_BEGIN();
  RUN_TEST(test_fib_fast_0);
  RUN_TEST(test_fib_fast_1);
  RUN_TEST(test_fib_fast_10);
  RUN_TEST(test_repeat);
  RUN_TEST(test_join_empty);
  RUN_TEST(test_join_single);
  RUN_TEST(test_join_many);
  RUN_TEST(test_find_quoted_1);
  RUN_TEST(test_find_quoted_2);
  RUN_TEST(test_find_quoted_3);
  return UNITY_END();
}
