#include "../unity/unity.h"
#include <stdlib.h>
#include <string.h>

/** Definitions ***************************************************************/

int add_3(int x) { return x + 3; }
void add_assign_3(int *x) { *x += 3; }

struct Point {
  int x;
  int y;
};

struct Point add_point(struct Point p1, struct Point p2) {
  struct Point p;
  p.x = p1.x + p2.x;
  p.y = p1.y + p2.y;
  return p;
}

void add_assign_point(struct Point *p1, struct Point p2) {
  p1->x += p2.x;
  p1->y += p2.y;
}

const char *find_char(char c, const char *s) {
  while (*s != 0) {
    if (*s == c) {
      return s;
    }
    s += 1;
  }
  return NULL;
}

/** Tests *********************************************************************/

void test_add_3(void) { TEST_ASSERT_EQUAL_INT(5, add_3(2)); }

void test_add_assign_3(void) {
  int x = 2;
  add_assign_3(&x);
  TEST_ASSERT_EQUAL_INT(5, x);
}

void test_add_point(void) {
  struct Point p1 = {1, 2};
  struct Point p2 = {10, 20};
  struct Point p3 = add_point(p1, p2);
  TEST_ASSERT_EQUAL_INT(11, p3.x);
  TEST_ASSERT_EQUAL_INT(22, p3.y);
}

void test_add_assign_point(void) {
  struct Point p1 = {1, 2};
  struct Point p2 = {10, 20};
  add_assign_point(&p1, p2);
  TEST_ASSERT_EQUAL_INT(11, p1.x);
  TEST_ASSERT_EQUAL_INT(22, p1.y);
}

/* According to the unity documentation the test for integers should also be
 * used for pointers.
 */

void test_find_char_success(void) {
  const char *s = "foo";
  TEST_ASSERT_EQUAL(s + 1, find_char('o', s));
}

void test_find_char_failure(void) {
  const char *s = "foo";
  TEST_ASSERT_EQUAL(NULL, find_char('z', s));
}

/** Main **********************************************************************/

void setUp(void) {}
void tearDown(void) {}

int main(void) {
  UNITY_BEGIN();
  RUN_TEST(test_add_3);
  RUN_TEST(test_add_assign_3);
  RUN_TEST(test_add_point);
  RUN_TEST(test_add_assign_point);
  RUN_TEST(test_find_char_success);
  RUN_TEST(test_find_char_failure);
  return UNITY_END();
}
